{------------------------------------------------------------------------------}
{
   Unit Name: tcpipform
   Purpose  : To give TCP/IP connections between machines
   Author   : Vesa Lappalainen
   Date     : 20.11.1999
   Changed  : 14.10.2001
    - muutettu kyttmn Indy-komponentteja
        IdTCPClient
        IdTCPServer
      entisten ServerSocket ja ClientSocket tilalla (tytyy tunnustaa
      ett Socket-komponentit olivta monin verroihn helpompia kytt).
      Nin saatu toimimaan sek VCL ett CLX.

    Kntmiseksi laita Search-path-kohtaan n:\kave\k2000\kcomp
    (tavara lytyy mys http://www.mit.jyu.fi/~vesal/kave/k2000/kcomp/)
    ja tuloshakemistot paikkaan, johon voit kirjoittaa.

   ToDo     :
    1) cr:n ksittelyn voisi siirt suoraan TCP/IP-komponenttiin,
       eli haluttaessa voisi automaattisesti list lhetyksiin cr
       ja jos tulee rivi, jossa on cr tekstin sisll, tmn voisi
       antaa komponentin kyttjlle useana palana
    2) voisi tehd oikean raahattavan komponentin, joka itse loisi
       tmn formin

    Tarkoitus on tarjota komponentti, joka voi toimia samalla sek
    palvelimena, ett asiakkaana.

    Jos asiakas saa merkkijonon, se vlitetn kaikille palveluun
    liittyneille (jos niit on).  Samoin jos palvelin saa merkkijonon,
    vlitt se merkkijonon edelleen kaikille palveluun liittyneille
    (paitsi lhettjlle) ja mys sille palvelimelle, jonka asiakkaana
    ollaan (jos ollaan).
    Tuli merkkijono kenelt tahansa, se vlitetn mys tiedoksi tmn
    komponentin kyttjlle.

    Tm komponentti tarjoaa mys SendInformation-metodin, mill komponentin
    kyttj voi lhett merkkijonon kaikille asiakkaille ja sille
    palvelimelle, johon komponentti on liittynyt asiakkaaksi.

    On mys metodit Listen palvelimen kynnistmiseksi ja Connect
    asiakkaaksi liittymiseksi.

    Vastaavasti CloseServer ja CloseClient.

    Yksinkertaisimmillaan tm lomake liitetn projektiin ja
    jossakin kohti koodia nytetn lomake, jotta asetuksia psee
    vaihtamaan.  Jos valmiit asetukset kelpaavat, ttkn ei tarvitse tehd
    (ajetaan jossakin tcpipform.Listen tai tcpipform.Connect).
    Mieluummin tietysti lomake luodaan dynaamisesti.

    Sitten ksitelln tapahtuma OnGetInformation.
    Katso esimerkki tcpiptest.pas.
}
{------------------------------------------------------------------------------}

unit tcpipformIndy;

interface

uses
  SysUtils,
  Classes,
{$ifdef CLX}
  QGraphics, QControls, QForms, QDialogs, QStdCtrls, QExtCtrls,QComCtrls,
{$else}
  Windows, Messages,
  Graphics, Controls, Forms, Dialogs, StdCtrls,  ExtCtrls,ComCtrls,
{$endif}
  IdTCPConnection, IdTCPClient, IdBaseComponent, IdComponent,
  IdTCPServer, kEditPnl, IdAntiFreezeBase, IdAntiFreeze;


type
  TTCPIPGetInformationEvent = function (const s:string):boolean of object;

  TFormTCPIP = class;

  TClientThread = class(TThread)
  private
    form:TFormTCPIP;
    data :string;
    procedure Send;
  public
    constructor Create(form:TFormTCPIP); virtual;
    procedure Execute; override;
  end;


  TFormTCPIP = class(TForm)
    PanelServer: TPanel;
    PanelClient: TPanel;
    GroupBoxServerSettings: TGroupBox;
    GroupBoxClientSettings: TGroupBox;
    MemoClientError: TMemo;
    MemoServerError: TMemo;
    NotebookServer: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    ButtonServerListen: TButton;
    ButtonServerClose: TButton;
    LabelConnectionsText: TLabel;
    LabelConnections: TLabel;
    NotebookClient: TPageControl;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    ButtonClientConnect: TButton;
    ButtonClientClose: TButton;
    Server: TIdTCPServer;
    Client: TIdTCPClient;
    EditServerPort: TEditPanel;
    EditServerPortEdit: TEdit;
    EditHost: TEditPanel;
    EditHostEdit: TEdit;
    EditClientPort: TEditPanel;
    EditClientPortEdit: TEdit;
    procedure ButtonServerListenClick(Sender: TObject);
    procedure ButtonClientConnectClick(Sender: TObject);
    procedure ButtonServerCloseClick(Sender: TObject);
    procedure ButtonClientCloseClick(Sender: TObject);
    procedure ClientConnected(Sender: TObject);
    procedure ClientDisconnected(Sender: TObject=nil);
    procedure ServerConnect(AThread: TIdPeerThread);
    procedure ServerDisconnect(AThread: TIdPeerThread);
    procedure ServerExecute(AThread: TIdPeerThread);
    procedure ClientStatus(axSender: TObject; const axStatus: TIdStatus;
      const asStatusText: String);
    procedure FormDestroy(Sender: TObject);
    procedure ServerStatus(axSender: TObject; const axStatus: TIdStatus;
      const asStatusText: String);
  private
    thread : TClientThread;
    serverData : string;
    FServerConnections : integer;
    FOnGetInformation: TTCPIPGetInformationEvent;
    procedure UpdateConnections(i:integer=0); virtual;
    procedure ClientThreadTermiante(Sender: TObject);
    procedure sendServerData;
    procedure IncServerConnections;
    procedure DecServerConnections;
    function GetClientHost: string;
    function GetClientPort: integer;
    function GetServerPort: integer;
    procedure SetClientHost(const Value: string);
    procedure SetClientPort(const Value: integer);
    procedure SetServerPort(const Value: integer);
    function GetClientConnections: integer;
    function GetTotalConnections: integer;
  public
    procedure SendInformation(const s:string;WhoNot: TIdPeerThread=nil;alsoClient:boolean=true); virtual;
    procedure ClientConnect; virtual;
    procedure ServerListen; virtual;
    procedure CloseServer; virtual;
    procedure CloseClient; virtual;
    property ServerConnections : integer read FServerConnections;
    property ClientConnections : integer read GetClientConnections;
    property TotalConnections : integer read GetTotalConnections;
  published
    property OnGetInformation : TTCPIPGetInformationEvent read FOnGetInformation write FOnGetInformation;
    property ServerPort : integer read GetServerPort write SetServerPort;
    property ClientPort : integer read GetClientPort write SetClientPort;
    property ClientHost : string read GetClientHost write SetClientHost;
  end;

var
  FormTCPIP: TFormTCPIP;

implementation

{$ifdef CLX}
{r *.xfm} // MN: ei toimi samaan aikaan vcl-projektissa...
{$else}
{$r *.dfm}
{$endif}


const pgConnected = 1;
      pgDisConnected = 0;

{ TFormTCPIP }

procedure TFormTCPIP.SendInformation(const s:string;WhoNot: TIdPeerThread;alsoClient:boolean);
var i:integer; list : TList; t: TIdPeerThread;
begin
  if ( Client.Connected ) and ( alsoClient ) then Client.Write(s);
  try
    List := Server.Threads.LockList;
    for i:=0 to List.Count-1 do begin
      t := List[i];
      if ( t = WhoNot ) then continue;
      t.Connection.Write(s);
    end;
  finally
    Server.Threads.UnLockList;
  end;
end;

procedure TFormTCPIP.ServerListen;
begin
  Server.Active := false;
  Server.DefaultPort := ServerPort;
  if ( Server.Bindings.Count > 0 ) then  // Bugi Indyss kun ei tee tt???
    Server.Bindings[0].Port := Server.DefaultPort;
  Server.OnDisconnect := ServerDisconnect;
  Server.Active := true;
  NotebookServer.ActivePageIndex := pgConnected;
  UpdateConnections;
end;

procedure TFormTCPIP.ButtonServerListenClick(Sender: TObject);
begin
  ServerListen;
end;

procedure TFormTCPIP.ClientConnect;
begin
  Client.Disconnect;
  Client.Port := ClientPort;
  Client.Host := EditHost.Text;
  Client.Connect;
end;

procedure TFormTCPIP.ButtonClientConnectClick(Sender: TObject);
begin
  ClientConnect;
end;

procedure TFormTCPIP.CloseServer;
begin
  Server.OnDisconnect := nil;
  NotebookServer.ActivePageIndex := pgDisConnected;
  Server.Active := false;
  UpdateConnections;
end;

procedure TFormTCPIP.ButtonServerCloseClick(Sender: TObject);
begin
  CloseServer;
end;

procedure TFormTCPIP.CloseClient;
begin
  NoteBookClient.ActivePageIndex := pgDisConnected;
  Client.Disconnect;
end;


procedure TFormTCPIP.ButtonClientCloseClick(Sender: TObject);
begin
  CloseClient;
end;

procedure TFormTCPIP.UpdateConnections(i:integer);
var list : TList;
begin
  try
    FServerConnections := 0;
    List := Server.Threads.LockList;
    FServerConnections:=List.Count;
  finally
    Server.Threads.UnLockList;
  end;
  FServerConnections := FServerConnections + i;
  LabelConnections.Caption := IntToStr(FServerConnections);
end;

procedure TFormTCPIP.ClientConnected(Sender: TObject);
begin
  NoteBookClient.ActivePageIndex := pgConnected;
  thread := TClientThread.Create(self);
end;

procedure TFormTCPIP.ClientThreadTermiante(Sender: TObject);
begin
  FreeAndNil(thread);
end;

procedure TFormTCPIP.ClientDisconnected(Sender: TObject);
begin
  NoteBookClient.ActivePageIndex := pgDisConnected;
  FreeAndNil(thread);
end;

procedure TFormTCPIP.FormDestroy(Sender: TObject);
begin
  FreeAndNil(thread);
end;

procedure TFormTCPIP.IncServerConnections;
begin
  UpdateConnections;
end;

procedure TFormTCPIP.DecServerConnections;
begin
  UpdateConnections(-1);
end;

procedure TFormTCPIP.ServerConnect(AThread: TIdPeerThread);
begin
  AThread.Synchronize(IncServerConnections);
end;

procedure TFormTCPIP.ServerDisconnect(AThread: TIdPeerThread);
begin
  AThread.Synchronize(DecServerConnections);
end;

procedure TFormTCPIP.sendServerData();
begin
  if ( Assigned(OnGetInformation) ) then OnGetInformation(serverData);
end;

procedure TFormTCPIP.ServerExecute(AThread: TIdPeerThread);
var s:string;
begin
// mn: delphi 7 indy-komponenteissa ei ole kutsua.
//  if ( not AThread.Connection.Server.Binding.Readable(1) ) then exit;
  s := AThread.Connection.CurrentReadBuffer;
  if ( s = '' ) then exit;
  serverData := s;
  AThread.Synchronize(sendServerData);
  SendInformation(s,AThread,true);
end;

procedure TFormTCPIP.ServerStatus(axSender: TObject;
  const axStatus: TIdStatus; const asStatusText: String);
var s:string;
begin
// Never comes here???
  s := asStatusText;
  if ( s = '' ) then exit;
  MemoServerError.Lines.Add(s);
end;

procedure TFormTCPIP.ClientStatus(axSender: TObject;
  const axStatus: TIdStatus; const asStatusText: String);
var s:string;
begin
  s := asStatusText;
  if ( s = '' ) then exit;
  MemoClientError.Lines.Add(s);
  if axStatus = hsDisconnecting then ClientDisconnected;
end;


function TFormTCPIP.GetClientHost: string;
begin
  Result := EditHost.AsString;
end;

function TFormTCPIP.GetClientPort: integer;
begin
  Result := StrToIntDef(EditClientPort.AsString,0);
end;

function TFormTCPIP.GetServerPort: integer;
begin
  Result := StrToIntDef(EditServerPort.AsString,0);
end;

procedure TFormTCPIP.SetClientHost(const Value: string);
begin
  EditHost.AsString := Value;
end;

procedure TFormTCPIP.SetClientPort(const Value: integer);
begin
  EditClientPort.AsString := IntToStr(Value);
end;

procedure TFormTCPIP.SetServerPort(const Value: integer);
begin
  EditServerPort.AsString := IntToStr(Value);
end;

function TFormTCPIP.GetClientConnections: integer;
begin
  Result := 0;
  if ( not Client.Connected ) then exit;
  Result := 1;
end;

function TFormTCPIP.GetTotalConnections: integer;
begin
  Result := ServerConnections + ClientConnections;
end;

{ TClientThread }

constructor TClientThread.Create(form:TFormTCPIP);
begin
  inherited Create(false);
//  self.FreeOnTerminate := true;
  self.form := form;
  self.OnTerminate := form.ClientThreadTermiante;
end;

procedure TClientThread.Send;
begin
  if ( Assigned(form.OnGetInformation) ) then form.OnGetInformation(data);
  form.SendInformation(data,nil,false);
end;

procedure TClientThread.Execute;
var s:string;
begin
  while ( form.Client.Connected ) and ( not Terminated ) do begin
    if ( not form.Client.IOHandler.Readable(1) ) then begin
      Sleep(1);
      continue;
    end;
(*    if ( not form.Client.Binding.Readable(1) ) then begin
      Sleep(1);
      continue;
    end;*)
    s := form.Client.CurrentReadBuffer;
    if ( s = '' ) then continue;
    data := s;
    Synchronize(Send);
  end;
end;


end.
